@@ -123,7 +123,7 @@ AddUserIDCommand::AddUserIDCommand(QAbstractItemView *v, KeyListController *c)
 }
 
 AddUserIDCommand::AddUserIDCommand(const GpgME::Key &key)
-    : Command(key, new Private(this, 0))
+    : Command(key, new Private(this, nullptr))
 {
     d->init();
 }
