@@ -1,7 +1,8 @@
 /*  crypto/gui/signencryptwidget.cpp
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2016 Intevation GmbH
+    Copyright (c) 2016 by Bundesamt für Sicherheit in der Informationstechnik
+    Software engineering by Intevation GmbH
 
     Kleopatra is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,6 +49,8 @@
 #include <Libkleo/KeySelectionCombo>
 #include <Libkleo/KeyListSortFilterProxyModel>
 
+#include <utils/gnupg-helper.h>
+
 #include <KLocalizedString>
 #include <KConfigGroup>
 #include <KSharedConfig>
@@ -277,6 +280,28 @@ QVector <Key> SignEncryptWidget::recipients() const
     return ret;
 }
 
+bool SignEncryptWidget::isDeVsAndValid() const
+{
+    if (!signKey().isNull()
+        && (!IS_DE_VS(signKey()) || keyValidity(signKey()) < GpgME::UserID::Validity::Full)) {
+        return false;
+    }
+
+    if (!selfKey().isNull()
+        && (!IS_DE_VS(selfKey())
+            || keyValidity(selfKey()) < GpgME::UserID::Validity::Full)) {
+        return false;
+    }
+
+    for (const auto &key: recipients()) {
+	if (!IS_DE_VS(key) || keyValidity(key) < GpgME::UserID::Validity::Full) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
 void SignEncryptWidget::updateOp()
 {
     const Key sigKey = signKey();
@@ -292,10 +317,8 @@ void SignEncryptWidget::updateOp()
     } else {
         newOp = QString();
     }
-    if (newOp != mOp) {
-        mOp = newOp;
-        Q_EMIT operationChanged(mOp);
-    }
+    mOp = newOp;
+    Q_EMIT operationChanged(mOp);
     Q_EMIT keysChanged();
 }
 
