@@ -3,6 +3,8 @@
 
     This file is part of Kleopatra, the KDE keymanager
     Copyright (c) 2008 Klarälvdalens Datakonsult AB
+    2016 by Bundesamt für Sicherheit in der Informationstechnik
+    Software engineering by Intevation GmbH
 
     Kleopatra is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -115,6 +117,11 @@ void TaskCollection::Private::taskStarted()
     assert(m_tasks.find(task->id()) != m_tasks.end());
     Q_EMIT q->started(m_tasks[task->id()]);
     calculateAndEmitProgress(); // start Knight-Rider-Mode right away (gpgsm doesn't report _any_ progress).
+    if (m_doneEmitted) {
+        // We are not done anymore, one task restarted.
+        m_nCompleted--;
+        m_doneEmitted = false;
+    }
 }
 
 void TaskCollection::Private::calculateAndEmitProgress()
