@@ -69,11 +69,9 @@ static QString pgpLabel(const QString &attr)
 {
     if (attr == QLatin1String("NAME")) {
         return i18n("Name");
-    }
-    if (attr == QLatin1String("COMMENT")) {
+    } else if (attr == QLatin1String("COMMENT")) {
         return i18n("Comment");
-    }
-    if (attr == QLatin1String("EMAIL")) {
+    } else if (attr == QLatin1String("EMAIL")) {
         return i18n("EMail");
     }
     return QString();
@@ -198,7 +196,7 @@ private:
 
             QMap<int, Line> lines;
 
-            Q_FOREACH (const QString &rawKey, attrOrder) {
+            for (const QString &rawKey : attrOrder) {
                 const QString key = rawKey.trimmed().toUpper();
                 const QString attr = attributeFromKey(key);
                 if (attr.isEmpty()) {
@@ -212,7 +210,7 @@ private:
                 const QString regex = config.readEntry(attr + QLatin1String("_regex"));
 
                 int row;
-                QValidator *validator = 0;
+                QValidator *validator = nullptr;
                 if (attr == QLatin1String("EMAIL")) {
                     validator = regex.isEmpty() ? Validation::email() : Validation::email(QRegExp(regex));
                     row = row_index_of(emailLE, gridLayout);
@@ -290,7 +288,7 @@ static bool has_intermediate_input(const QLineEdit *le)
 
 static bool requirementsAreMet(const QVector<Line> &list, QString &error)
 {
-    Q_FOREACH (const Line &line, list) {
+    for (const Line &line : list) {
         const QLineEdit *le = line.edit;
         if (!le) {
             continue;
@@ -354,7 +352,7 @@ void AddUserIDDialog::Private::slotUserIDChanged()
         parts.push_back(QLatin1Char('<') + email + QLatin1Char('>'));
     }
 
-    ui.resultLB->setText(parts.join(QStringLiteral(" ")));
+    ui.resultLB->setText(parts.join(QLatin1Char(' ')));
 }
 
 #include "moc_adduseriddialog.cpp"
