@@ -3,7 +3,8 @@
 
     This file is part of Kleopatra, the KDE keymanager
     Copyright (c) 2008 Klarälvdalens Datakonsult AB
-                  2016 Intevation GmbH
+    2016, 2017 by Bundesamt für Sicherheit in der Informationstechnik
+    Software engineering by Intevation GmbH
 
     Kleopatra is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -59,8 +60,10 @@
 
 #include <QGpgME/KeyGenerationJob>
 #include <QGpgME/Protocol>
+#include <QGpgME/CryptoConfig>
 
 #include <gpgme++/global.h>
+#include <gpgme++/gpgmepp_version.h>
 #include <gpgme++/keygenerationresult.h>
 
 #include <KConfigGroup>
@@ -208,7 +211,7 @@ static void set_curve(QComboBox *cb, const QString &curve)
     if (!cb) {
         return;
     }
-    const int idx = cb->findData(curve);
+    const int idx = cb->findText(curve);
     if (idx < 0) {
         // Can't happen as we don't have them configurable.
         qCWarning(KLEOPATRA_LOG) << "curve " << curve << " not allowed";
@@ -224,6 +227,64 @@ static QString get_curve(const QComboBox *cb)
     return cb->currentText();
 }
 
+// Extract the algo information from default_pubkey_algo format
+//
+// and put it into the return values size, algo and curve.
+//
+// Values look like:
+// RSA-2048
+// rsa2048/cert,sign+rsa2048/enc
+// brainpoolP256r1+brainpoolP256r1
+static void parseAlgoString(const QString &algoString, int *size, Subkey::PubkeyAlgo *algo, QString &curve)
+{
+    const auto split = algoString.split(QLatin1Char('/'));
+    bool isEncrypt = split.size() == 2 && split[1].contains(QStringLiteral("enc"));
+
+    // Normalize
+    const auto lowered = split[0].toLower().remove(QLatin1Char('-'));
+    if (!algo || !size) {
+        return;
+    }
+    *algo = Subkey::AlgoUnknown;
+    if (lowered.startsWith(QStringLiteral("rsa"))) {
+        *algo = Subkey::AlgoRSA;
+    } else if (lowered.startsWith(QStringLiteral("dsa"))) {
+        *algo = Subkey::AlgoDSA;
+    } else if (lowered.startsWith(QStringLiteral("elg"))) {
+        *algo = Subkey::AlgoELG;
+    }
+
+    if (*algo != Subkey::AlgoUnknown) {
+        bool ok;
+        *size = lowered.right(lowered.size() - 3).toInt(&ok);
+        if (!ok) {
+            qCWarning(KLEOPATRA_LOG) << "Could not extract size from: " << lowered;
+            *size = 2048;
+        }
+        return;
+    }
+
+    // Now the ECC Algorithms
+    if (lowered.startsWith(QStringLiteral("ed25519"))) {
+        // Special handling for this as technically
+        // this is a cv25519 curve used for EDDSA
+        curve = split[0];
+        *algo = Subkey::AlgoEDDSA;
+        return;
+    }
+
+    if (lowered.startsWith(QStringLiteral("cv25519")) ||
+        lowered.startsWith(QStringLiteral("nist")) ||
+        lowered.startsWith(QStringLiteral("brainpool")) ||
+        lowered.startsWith(QStringLiteral("secp"))) {
+        curve = split[0];
+        *algo = isEncrypt ? Subkey::AlgoECDH : Subkey::AlgoECDSA;
+        return;
+    }
+
+    qCWarning(KLEOPATRA_LOG) << "Failed to parse default_pubkey_algo:" << algoString;
+}
+
 Q_DECLARE_METATYPE(GpgME::Subkey::PubkeyAlgo)
 namespace Kleo
 {
@@ -276,7 +337,6 @@ protected:
 
     FIELD(QString, name)
     FIELD(QString, email)
-    FIELD(QString, comment)
     FIELD(QString, dn)
 
     FIELD(Subkey::PubkeyAlgo, keyType)
@@ -594,6 +654,7 @@ private Q_SLOTS:
 private:
     void fillKeySizeComboBoxen();
     void loadDefaultKeyType();
+    void loadDefaultGnuPGKeyType();
     void updateWidgetVisibility();
 
 private:
@@ -637,7 +698,7 @@ public:
             ui.x509CLB->isChecked() ? CMS : UnknownProtocol;
     }
 
-    void initializePage() Q_DECL_OVERRIDE {
+    void initializePage() override {
         if (!initialized)
         {
             connect(ui.pgpCLB,  &QAbstractButton::clicked, wizard(), &QWizard::next, Qt::QueuedConnection);
@@ -646,7 +707,7 @@ public:
         initialized = true;
     }
 
-    bool isComplete() const Q_DECL_OVERRIDE
+    bool isComplete() const override
     {
         return protocol() != UnknownProtocol;
     }
@@ -688,16 +749,15 @@ public:
         registerField(QStringLiteral("dn"), ui.resultLE);
         registerField(QStringLiteral("name"), ui.nameLE);
         registerField(QStringLiteral("email"), ui.emailLE);
-        registerField(QStringLiteral("comment"), ui.commentLE);
         updateForm();
     }
 
-    bool isComplete() const Q_DECL_OVERRIDE;
-    void initializePage() Q_DECL_OVERRIDE {
+    bool isComplete() const override;
+    void initializePage() override {
         updateForm();
         dialog.setProtocol(pgp() ? OpenPGP : CMS);
     }
-    void cleanupPage() Q_DECL_OVERRIDE {
+    void cleanupPage() override {
         saveValues();
     }
 
@@ -738,7 +798,7 @@ public:
         setButtonText(QWizard::CommitButton, i18nc("@action", "Create Key"));
     }
 
-    void initializePage() Q_DECL_OVERRIDE {
+    void initializePage() override {
         slotShowDetails();
     }
 
@@ -769,12 +829,12 @@ public:
         ui.setupUi(this);
     }
 
-    bool isComplete() const Q_DECL_OVERRIDE
+    bool isComplete() const override
     {
         return !job;
     }
 
-    void initializePage() Q_DECL_OVERRIDE {
+    void initializePage() override {
         startJob();
     }
 
@@ -866,7 +926,7 @@ public:
         registerField(QStringLiteral("fingerprint"), le);
     }
 
-    void initializePage() Q_DECL_OVERRIDE {
+    void initializePage() override {
         const bool error = isError();
 
         if (error)
@@ -913,7 +973,7 @@ public:
         initialized = true;
     }
 
-    void cleanupPage() Q_DECL_OVERRIDE {
+    void cleanupPage() override {
         setButtonVisible(QWizard::CancelButton, true);
     }
 
@@ -922,7 +982,7 @@ public:
         return !ui.errorTB->document()->isEmpty();
     }
 
-    bool isComplete() const Q_DECL_OVERRIDE
+    bool isComplete() const override
     {
         return !isError();
     }
@@ -1187,9 +1247,6 @@ static QString pgpLabel(const QString &attr)
     if (attr == QLatin1String("NAME")) {
         return i18n("Name");
     }
-    if (attr == QLatin1String("COMMENT")) {
-        return i18n("Comment");
-    }
     if (attr == QLatin1String("EMAIL")) {
         return i18n("EMail");
     }
@@ -1261,7 +1318,7 @@ void EnterDetailsPage::registerDialogPropertiesAsFields()
 
 void EnterDetailsPage::saveValues()
 {
-    Q_FOREACH (const Line &line, lineList) {
+    for (const Line &line : qAsConst(lineList)) {
         savedValues[ attributeFromKey(line.attr) ] = line.edit->text().trimmed();
     }
 }
@@ -1282,11 +1339,6 @@ void EnterDetailsPage::clearForm()
     ui.emailLB->hide();
     ui.emailRequiredLB->hide();
 
-    ui.commentLE->hide();
-    ui.commentLE->clear();
-    ui.commentLB->hide();
-    ui.commentRequiredLB->hide();
-
     ui.addEmailToDnCB->hide();
 }
 
@@ -1357,7 +1409,7 @@ void EnterDetailsPage::updateForm()
     QStringList attrOrder = config.readEntry(pgp() ? "OpenPGPAttributeOrder" : "DNAttributeOrder", QStringList());
     if (attrOrder.empty()) {
         if (pgp()) {
-            attrOrder << QStringLiteral("NAME!") << QStringLiteral("EMAIL!") << QStringLiteral("COMMENT");
+            attrOrder << QStringLiteral("NAME!") << QStringLiteral("EMAIL!");
         } else {
             attrOrder << QStringLiteral("CN!") << QStringLiteral("L") << QStringLiteral("OU") << QStringLiteral("O!") << QStringLiteral("C!") << QStringLiteral("EMAIL!");
         }
@@ -1366,7 +1418,6 @@ void EnterDetailsPage::updateForm()
     QList<QWidget *> widgets;
     widgets.push_back(ui.nameLE);
     widgets.push_back(ui.emailLE);
-    widgets.push_back(ui.commentLE);
 
     QMap<int, Line> lines;
 
@@ -1400,12 +1451,6 @@ void EnterDetailsPage::updateForm()
                 validator = regex.isEmpty() ? Validation::pgpName() : Validation::pgpName(QRegExp(regex));
             }
             row = row_index_of(ui.nameLE, ui.gridLayout);
-        } else if (attr == QLatin1String("COMMENT")) {
-            if (!pgp()) {
-                continue;
-            }
-            validator = regex.isEmpty() ? Validation::pgpComment() : Validation::pgpComment(QRegExp(regex));
-            row = row_index_of(ui.commentLE, ui.gridLayout);
         } else {
             known = false;
             row = add_row(ui.gridLayout, &dynamicWidgets);
@@ -1473,7 +1518,7 @@ QString EnterDetailsPage::pgpUserID() const
     return Formatting::prettyNameAndEMail(OpenPGP, QString(),
                                           ui.nameLE->text().trimmed(),
                                           ui.emailLE->text().trimmed(),
-                                          ui.commentLE->text().trimmed());
+                                          QString());
 }
 
 static bool has_intermediate_input(const QLineEdit *le)
@@ -1631,11 +1676,7 @@ QString OverviewPage::i18nFormatGnupgKeyParms(bool details) const
         s         << Row<        >(i18n("Name:"),              name());
     }
     s             << Row<        >(i18n("Email Address:"),     email());
-    if (pgp()) {
-        if (!comment().isEmpty()) {
-            s     << Row<        >(i18n("Comment:"),           comment());
-        }
-    } else {
+    if (!pgp()) {
         s         << Row<        >(i18n("Subject-DN:"),        DN(dn()).dn(QStringLiteral(",<br>")));
     }
     if (details) {
@@ -1723,9 +1764,6 @@ QString KeyCreationPage::createGnupgKeyParms() const
     if (pgp()) {
         s << "name-real:     " << name()                   << endl;
         s << "name-email:    " << email()                  << endl;
-        if (!comment().isEmpty()) {
-            s << "name-comment:  " << comment()            << endl;
-        }
     } else {
         s << "name-dn:       " << dn()                     << endl;
         s << "name-email:    " << encode_email(email())    << endl;
@@ -1783,6 +1821,68 @@ void AdvancedSettingsDialog::fillKeySizeComboBoxen()
     ui.ecdsaKeyCurvesCB->addItems(curveNames);
 }
 
+// Try to load the default key type from GnuPG
+void AdvancedSettingsDialog::loadDefaultGnuPGKeyType()
+{
+    const auto conf = QGpgME::cryptoConfig();
+    if (!conf) {
+        qCWarning(KLEOPATRA_LOG) << "Failed to obtain cryptoConfig.";
+        return;
+    }
+    const auto entry = conf->entry(protocol == CMS ? QStringLiteral("gpgsm") : QStringLiteral("gpg"),
+                                   QStringLiteral("Configuration"),
+                                   QStringLiteral("default_pubkey_algo"));
+    if (!entry) {
+        // GnuPG does not support that option.
+        return;
+    }
+
+    qCDebug(KLEOPATRA_LOG) << "Have default key type: " << entry->stringValue();
+
+    // Format is <primarytype>[/usage]+<subkeytype>[/usage]
+    const auto split = entry->stringValue().split(QLatin1Char('+'));
+    int size = 0;
+    Subkey::PubkeyAlgo algo = Subkey::AlgoUnknown;
+    QString curve;
+
+    parseAlgoString(split[0], &size, &algo, curve);
+    if (algo == Subkey::AlgoUnknown) {
+        setSubkeyType(Subkey::AlgoRSA);
+        return;
+    }
+
+    setKeyType(algo);
+
+    if (is_rsa(algo) || is_elg(algo) || is_dsa(algo)) {
+        setKeyStrength(size);
+    } else {
+        setKeyCurve(curve);
+    }
+
+    if (split.size() == 2) {
+        auto algoString = split[1];
+        // If it has no usage we assume encrypt subkey
+        if (!algoString.contains(QLatin1Char('/'))) {
+            algoString += QStringLiteral("/enc");
+        }
+
+        parseAlgoString(algoString, &size, &algo, curve);
+
+        if (algo == Subkey::AlgoUnknown) {
+            setSubkeyType(Subkey::AlgoRSA);
+            return;
+        }
+
+        setSubkeyType(algo);
+
+        if (is_rsa(algo) || is_elg(algo)) {
+            setSubkeyStrength(size);
+        } else {
+            setSubkeyCurve(curve);
+        }
+    }
+}
+
 void AdvancedSettingsDialog::loadDefaultKeyType()
 {
 
@@ -1801,6 +1901,12 @@ void AdvancedSettingsDialog::loadDefaultKeyType()
     } else if (protocol == OpenPGP && keyType == QLatin1String("DSA+ELG")) {
         setKeyType(Subkey::AlgoDSA);
         setSubkeyType(Subkey::AlgoELG_E);
+#if GPGMEPP_VERSION > 0x10800
+        // GPGME 1.8.0 has a bug that makes the gpgconf engine
+        // return garbage so we don't load it for this
+    } else if (keyType.isEmpty() && engineIsVersion(2, 1, 17)) {
+        loadDefaultGnuPGKeyType();
+#endif
     } else {
         if (!keyType.isEmpty() && keyType != QLatin1String("RSA"))
             qCWarning(KLEOPATRA_LOG) << "invalid value \"" << qPrintable(keyType)
@@ -1862,6 +1968,7 @@ void AdvancedSettingsDialog::updateWidgetVisibility()
     if (protocol == CMS) {
         ui.encryptionCB->setEnabled(true);
         ui.rsaSubCB->setChecked(false);
+        ui.rsaKeyStrengthSubCB->setEnabled(false);
     }
     ui.expiryDE->setVisible(protocol == OpenPGP);
     ui.expiryCB->setVisible(protocol == OpenPGP);
