@@ -1,7 +1,8 @@
 /*  keycacheoverlay.h
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2016 Intevation GmbH
+    Copyright (c) 2016 by Bundesamt für Sicherheit in der Informationstechnik
+    Software engineering by Intevation GmbH
 
     Kleopatra is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -53,7 +54,7 @@ public:
     explicit KeyCacheOverlay(QWidget *baseWidget, QWidget *parent = nullptr);
 
 protected:
-    bool eventFilter(QObject *object, QEvent *event) Q_DECL_OVERRIDE;
+    bool eventFilter(QObject *object, QEvent *event) override;
 
 private:
     void reposition();
